/*
   This is an example sketch that shows how to toggle the display
   on and off at runtime to avoid screen burn-in.

   The sketch also demonstrates how to erase a previous value by re-drawing the
   older value in the screen background color prior to writing a new value in
   the same location. This avoids the need to call fillScreen() to erase the
   entire screen followed by a complete redraw of screen contents.

   Originally written by Phill Kelley. BSD license.
   Adapted for ST77xx by Melissa LeBlanc-Williams
*/

#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library for ST7735
#include <SPI.h>

#define TFT_CS        17
#define TFT_RST        1
#define TFT_DC         0


#define SerialDebugging true
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST);
// connect a push button between ground and...
const uint8_t   Button_pin              = 2; // GPIO2

// color definitions
const uint16_t  Display_Color_Black        = 0x0000;
const uint16_t  Display_Color_Blue         = 0x001F;
const uint16_t  Display_Color_Red          = 0xF800;
const uint16_t  Display_Color_Green        = 0x07E0;
const uint16_t  Display_Color_Cyan         = 0x07FF;
const uint16_t  Display_Color_Magenta      = 0xF81F;
const uint16_t  Display_Color_Yellow       = 0xFFE0;
const uint16_t  Display_Color_White        = 0xFFFF;

// The colors we actually want to use
uint16_t        Display_Text_Color         = Display_Color_White;
uint16_t        Display_Backround_Color    = Display_Color_Blue;

// assume the display is off until configured in setup()
bool            isDisplayVisible        = false;

// declare size of working string buffers. Basic strlen("d hh:mm:ss") = 10
const size_t    MaxString               = 16;

// the string being displayed on the SSD1331 (initially empty)
char oldTimeString[MaxString]           = { 0 };

// the interrupt service routine affects this
volatile bool   isButtonPressed         = false;

//---------------------------------
// interrupt service routine
//---------------------------------
void senseButtonPressed() {
  if (!isButtonPressed) {
    isButtonPressed = true;
  }
}
//---------------------------------

void displayUpTime() {
  // calculate seconds, truncated to the nearest whole second
  unsigned long upSeconds = millis() / 1000;
  // calculate days, truncated to nearest whole day
  unsigned long days = upSeconds / 86400;
  // the remaining hhmmss are
  upSeconds = upSeconds % 86400;
  // calculate hours, truncated to the nearest whole hour
  unsigned long hours = upSeconds / 3600;
  // the remaining mmss are
  upSeconds = upSeconds % 3600;
  // calculate minutes, truncated to the nearest whole minute
  unsigned long minutes = upSeconds / 60;
  // the remaining ss are
  upSeconds = upSeconds % 60;
  // allocate a buffer
  char newTimeString[MaxString] = { 0 };
  // construct the string representation
  sprintf(
    newTimeString,
    "%lu %02lu:%02lu:%02lu",
    days, hours, minutes, upSeconds
  );

  // has the time string changed since the last tft update?
  if (strcmp(newTimeString, oldTimeString) != 0) {
    // yes! home the cursor
    tft.setCursor(0, 0);
    // change the text color to the background color
    tft.setTextColor(Display_Backround_Color);
    // redraw the old value to erase
    tft.print(oldTimeString);
    // home the cursor
    tft.setCursor(0, 0);
    // change the text color to foreground color
    tft.setTextColor(Display_Text_Color);
    // draw the new time value
    tft.print(newTimeString);
    // and remember the new value
    strcpy(oldTimeString, newTimeString);
  }
}

void setup() {
  // button press pulls pin LOW so configure HIGH
  pinMode(Button_pin, INPUT_PULLUP);
  // use an interrupt to sense when the button is pressed
  attachInterrupt(digitalPinToInterrupt(Button_pin), senseButtonPressed, FALLING);
#if (SerialDebugging)
  Serial.begin(115200); while (!Serial); Serial.println();
#endif
  // settling time
  delay(2500);
  // ignore any power-on-reboot garbage
  isButtonPressed = false;
  // Use this initializer if using a 1.8" TFT screen:
  tft.initR(INITR_BLACKTAB);      // Init ST7735S chip, black tab
  tft.setSPISpeed(1000000);
  tft.setRotation(2);             // A tájolás beállítása
  // initialise the display
  tft.setFont();
  tft.fillScreen(Display_Backround_Color);
  tft.setTextColor(Display_Text_Color);
  tft.setTextSize(2);
  // the display is now on
  isDisplayVisible = true;
}

void loop() {
  // unconditional display, regardless of whether display is visible
  displayUpTime();
  // has the button been pressed?
  if (isButtonPressed) {
    // yes! toggle display visibility
    isDisplayVisible = !isDisplayVisible;
    // apply
    tft.enableDisplay(isDisplayVisible);
#if (SerialDebugging)
    Serial.print("button pressed @ ");
    Serial.print(millis());
    Serial.print(", display is now ");
    Serial.println((isDisplayVisible ? "ON" : "OFF"));
#endif
    // confirm button handled
    isButtonPressed = false;
  }
  // no need to be in too much of a hurry
  delay(100);
}
